/*!
 * @nuxt/cli v2.10.1 (c) 2016-2019

 * - All the amazing contributors
 * Released under the MIT License.
 * Website: https://nuxtjs.org
*/
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

const chalk = _interopDefault(require('chalk'));
const env = _interopDefault(require('std-env'));
const index$1 = require('./cli-index.js');
const consola = _interopDefault(require('consola'));
const prettyBytes = _interopDefault(require('pretty-bytes'));

function getMemoryUsage () {
  // https://nodejs.org/api/process.html#process_process_memoryusage
  const { heapUsed, rss } = process.memoryUsage();
  return { heap: heapUsed, rss }
}

function getFormattedMemoryUsage () {
  const { heap, rss } = getMemoryUsage();
  return `Memory usage: ${chalk.bold(prettyBytes(heap))} (RSS: ${prettyBytes(rss)})`
}

function showMemoryUsage () {
  consola.info(getFormattedMemoryUsage());
}

function showBanner (nuxt, showMemoryUsage = true) {
  if (env.test) {
    return
  }

  if (env.minimalCLI) {
    for (const listener of nuxt.server.listeners) {
      consola.info('Listening on: ' + listener.url);
    }
    return
  }

  const titleLines = [];
  const messageLines = [];

  // Name and version
  const { bannerColor } = nuxt.options.cli;
  titleLines.push(`${chalk[bannerColor].bold('Nuxt.js')} ${nuxt.constructor.version}`);

  // Running mode
  titleLines.push(`Running in ${nuxt.options.dev ? chalk.bold.blue('development') : chalk.bold.green('production')} mode (${chalk.bold(nuxt.options.mode)})`);

  if (showMemoryUsage) {
    titleLines.push(getFormattedMemoryUsage());
  }

  // Listeners
  for (const listener of nuxt.server.listeners) {
    messageLines.push(chalk.bold('Listening on: ') + chalk.underline.blue(listener.url));
  }

  // Add custom badge messages
  if (nuxt.options.cli.badgeMessages.length) {
    messageLines.push('', ...nuxt.options.cli.badgeMessages);
  }

  process.stdout.write(index$1.successBox(messageLines.join('\n'), titleLines.join('\n')));
}

exports.showBanner = showBanner;
exports.showMemoryUsage = showMemoryUsage;
