/*!
 * @nuxt/cli v2.10.1 (c) 2016-2019

 * - All the amazing contributors
 * Released under the MIT License.
 * Website: https://nuxtjs.org
*/
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

const command = require('./cli-command.js');
require('path');
require('@nuxt/config');
require('exit');
require('@nuxt/utils');
const chalk = _interopDefault(require('chalk'));
require('std-env');
require('wrap-ansi');
require('boxen');
const index$1 = require('./cli-index.js');
const consola = _interopDefault(require('consola'));
require('minimist');
require('hable');
require('esm');

async function listCommands () {
  const commandsOrder = ['dev', 'build', 'generate', 'start', 'help'];

  // Load all commands
  const _commands = await Promise.all(
    commandsOrder.map(cmd => command.getCommand(cmd))
  );

  let maxLength = 0;
  const commandsHelp = [];

  for (const command of _commands) {
    commandsHelp.push([command.usage, command.description]);
    maxLength = Math.max(maxLength, command.usage.length);
  }

  const _cmds = commandsHelp.map(([cmd, description]) => {
    const i = index$1.indent(maxLength + index$1.optionSpaces - cmd.length);
    return index$1.foldLines(
      chalk.green(cmd) + i + description,
      index$1.startSpaces + maxLength + index$1.optionSpaces * 2,
      index$1.startSpaces + index$1.optionSpaces
    )
  }).join('\n');

  const usage = index$1.foldLines(`Usage: nuxt <command> [--help|-h]`, index$1.startSpaces);
  const cmds = index$1.foldLines(`Commands:`, index$1.startSpaces) + '\n\n' + _cmds;

  process.stderr.write(index$1.colorize(`${usage}\n\n${cmds}\n\n`));
}

const help = {
  name: 'help',
  description: 'Shows help for <command>',
  usage: 'help <command>',
  options: {
    help: index$1.common.help,
    version: index$1.common.version
  },
  async run (cmd) {
    const [name] = cmd._argv;
    if (!name) {
      return listCommands()
    }
    const command$1 = await command.getCommand(name);

    if (!command$1) {
      consola.info(`Unknown command: ${name}`);
      return
    }

    command.NuxtCommand.from(command$1).showHelp();
  }
};

exports.default = help;
